import Mixin from '@ember/object/mixin'
import { get } from '@ember/object'
import { run } from '@ember/runloop'
import { warn } from '@ember/debug'

const capitalize = str => str.charAt(0).toUpperCase() + str.slice(1)

export default Mixin.create({
  documentEvents: [],

  didInsertElement() {
    this._super(...arguments)
    this._subscribeToEvents()
  },

  _subscribeToEvents() {
    run.scheduleOnce('afterRender', () => {
      get(this, 'documentEvents').forEach(event => {
        const eventHandler = `onDocument${capitalize(event)}`

        if (!this[eventHandler]) {
          warn(
            `[document-event] You are tracking ${event}, but there is no handler. ${eventHandler} was not found`,
          )
          return
        }

        this._handleEvent = this._handleEvent.bind(this, eventHandler)
        document.addEventListener(event, this._handleEvent, true)
      })
    })
  },

  _handleEvent(eventHandler, event) {
    run(() => {
      if (this[eventHandler]) this[eventHandler](event)
    })
  },

  willDestroyElement() {
    this._teardown()
  },

  _teardown() {
    get(this, 'documentEvents').forEach(event => {
      document.removeEventListener(event, this._handleEvent, true)
    })
  },
})
